Build requirements:
-------------------

- OCaml >= 4.11.0
- dune
- menhir
- cmdliner
- js_of_ocaml
- js_of_ocaml-tyxml
- ezjs_ace

Build instructions:
-------------------

Simply type ``make``. This should build the executable, and a
JavaScript version that can be embedded in a web page.

If needed, the command ``make clean`` cleans the whole build.

To run the program:
-------------------

Type the command ``dune exec -- cfa-simpl`` and add a file as
argument, that contains the program you want to analyze. You may add
option flags, that are documented using the ``dune exec -- cfa-simpl
--help`` command.

By default, the analysis that is performed is ``nabla``.

For example, you may type:

    dune exec -- cfa-simpl --nabla --refine-branches src/cfa/examples/mc91.simpl

This should print:

    { bools = ∅; ints = [91, +∞]; closures = ∅ }

which means that the program defined in the ``mc91.simpl`` file may
only produce an integer that is larger than 91. Documentation about
the syntax of programs and of the analyzer's outputs can be found in
the file ``src/cfa/demo/cfa.html``.

To run the tests:
-----------------

Simply type ``make test``. This runs all the tests, following the
configurations given in the file
``src/cfa/examples/generate_dune.ml``, that you can customize to
enable/disable some options.

The folder ``src/cfa/examples/`` contains several kinds of files:

- ``*.simpl`` files are example source files.
- ``*.expected-nabla`` files are expected outputs for the ``nabla``
  analysis.
- ``*.expected-global`` files are expected outputs for the ``global``
  analysis.
- ``*.expected-global-independent-attribute`` files are expected
  outputs for the ``global-independent-attribute`` analysis (not
  described in the paper).

If a difference between a computed result and an expected result is
detected, ``dune`` reports it automatically, and shows a diff.

To run the tests in your browser:
---------------------------------

Open the file ``src/cfa/demo/cfa.html`` in your preferred browser.
From your browser, you can interactively run the different analyzers
on the examples of your choice. The code runs locally in your browser.

To run the benchmarks:
----------------------

Simply type ``make bench``. _Warning_: it takes about 30 minutes on a
conventional 2020 laptop. This produces a file
``src/cfa/examples/cfa_bench_table.tex``, prints the difference (using
``diff``) with the reference file, prints ``Benchmarks: OK``. The
benchmarks do not measure performance, but instead report the sizes of
the search spaces that were explored. Those sizes are independent from
the quality of the implementation or the efficiency of the computer on
which the benchmarks are run. The benchmarks also report the results
of the analyses, as comments inside the TeX file (they would not fit
in the PDF table).

If there is a difference with the reference file, the procedure halts
with an error.

The messages

    INFO: Cycle found in global environment.

are harmless.

The benchmarks that are run are specified in the file
``src/cfa/examples/generate_bench.ml`` (see the end of that file for
the list of files to analyze and the selected options).

To produce a PDF report, type ``make pdf``. It will run the
``pdflatex`` command to produce the file
``src/cfa/examples/bench.pdf``.
