(** @author: Benoît Montagu <benoit.montagu@inria.fr>
    Copyright © Inria 2020-2021
*)

open Lang.Ast

module Make
    (Log: Logs.S)
    (Options: sig
       val debug: bool
     end)
    (Calls: Calls.S)
    (CtxtEnv: sig
       type t
       val empty: t
       val compare: t -> t -> int
       val pp: Format.formatter -> t -> unit
     end)
    (Abs: sig
       type t
       val bot: t
       val leq: t -> t -> bool
       val join: t -> t -> t
       val widen: t -> t -> t
       val pp: Format.formatter -> t -> unit
     end) :
sig
  module GEnv: sig
    type key = PP.t * Calls.t
    type value = Abs.t
    type t
    val bot: t
    val get: key -> t -> value
    val singleton: key -> value -> t
    val find_opt: key -> t -> value option
    val union: (key -> value -> value -> value option) -> t -> t -> t
    val merge: (key -> value option -> value option -> value option) -> t -> t -> t
    val iter: (key -> value -> unit) -> t -> unit
    val cardinal: t -> int
  end

  module BoolGEnv: sig
    type key = PP.t * Calls.t
    type value = bool
    type t
    val empty: t
    val find_opt: key -> t -> value option
    val add: key -> value -> t -> t
  end

  module GEnvAbs: sig
    type t = GEnv.t * Abs.t
    val bot: t
    val join: t -> t -> t
  end

  val cofix:
    ((bool -> Calls.t -> CtxtEnv.t -> Term.t -> GEnvAbs.t) ->
     (bool -> Calls.t -> CtxtEnv.t -> Term.t -> GEnvAbs.t)) ->
    (Calls.t -> CtxtEnv.t -> Term.t -> GEnvAbs.t * Memo.Statistics.t)

  val make_eval:
    print_stats:(GEnv.t -> unit) ->
    check_genv:(GEnv.t -> unit) ->
    (GEnv.t -> Calls.t -> CtxtEnv.t -> 'term -> GEnvAbs.t * Memo.Statistics.t) ->
    ('term -> Abs.t * Memo.Statistics.t)
end
