(** @author: Benoît Montagu <benoit.montagu@inria.fr>
    Copyright © Inria 2020
*)

module IntegerDomain: sig
  type t =
    | TwoPoints
    | Flat
    | Signs
    | Intervals

  val min: int
  val max: int
  val to_enum: t -> int
  val of_enum: int -> t option

  val parser: string -> (t, [`Msg of string]) result
  val printer: Format.formatter -> t -> unit
  val to_string: t -> string

  val default: t
  val examples: string list
end

module CallDomain: sig
  type t =
    | LastCallSites of int
    | CyclicCallSites of int

  val parser: string -> (t, [`Msg of string]) result
  val printer: Format.formatter -> t -> unit

  val default: t
  val examples: string list
end

module AnalysisKind: sig
  type t =
    | Nabla
    | Global
    | GlobalIndependentAttribute

  val min: int
  val max: int
  val to_enum: t -> int
  val of_enum: int -> t option

  val to_string: t -> string
end

module Options: sig
  type t = {
    refine_branches: bool;
    branch_is_context: bool;
    let_is_context: bool;
    no_env_restrict: bool;
    kill_unreachable: bool;
    debug: bool;
    integer_domain: IntegerDomain.t;
    call_domain: CallDomain.t;
    analysis_kind: AnalysisKind.t;
    share_integers_program_points: bool;
    share_variables_program_points: bool;
  }

  module type ANALYZER = sig
    module Abs : sig
      type t
      val pp: Format.formatter -> t -> unit
    end
    val eval: Lang.Ast.Term.t -> (Abs.t * Memo.Statistics.t)
  end

  val get_analysis_module: (module Logs.S) -> t -> (module ANALYZER)
end

val cfa_print: Options.t -> string -> unit

type package =
  | Pack: 'res * (Format.formatter -> 'res -> unit) * Memo.Statistics.t -> package

val cfa_value: Options.t -> string -> package
